@@ -1,7 +1,7 @@
 /*
  * gpomme - GTK application for use with pommed
  *
- * $Id: conffile.c 295 2007-03-08 17:48:00Z jblache $
+ * $Id: conffile.c 378 2007-11-16 21:50:10Z jblache $
  *
  * Copyright (C) 2007 Julien BLACHE <jb@jblache.org>
  * Copyright (C) 2007 daniel g. siegel <dgsiegel@gmail.com>
@@ -33,8 +33,12 @@
 #include <dirent.h>
 #include <libintl.h>
 
-#include <linux/inotify.h>
-#include "inotify-syscalls.h"
+#ifndef NO_SYS_INOTIFY_H
+# include <sys/inotify.h>
+#else
+# include <linux/inotify.h>
+# include "inotify-syscalls.h"
+#endif
 
 #include <confuse.h>
 
