@@ -1,7 +1,7 @@
 /*
  * gpomme - GTK application for use with pommed
  *
- * $Id: gpomme.c 341 2007-07-26 15:49:44Z jblache $
+ * $Id: gpomme.c 413 2007-12-04 11:15:51Z jblache $
  *
  * Copyright (C) 2006 Soeren SONNENBURG <debian@nn7.de>
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
@@ -30,21 +30,31 @@
 #include <unistd.h>
 #include <signal.h>
 
-#include <linux/inotify.h>
-#include "inotify-syscalls.h"
+#include <sys/types.h>
+#include <sys/wait.h>
+
+#ifndef NO_SYS_INOTIFY_H
+# include <sys/inotify.h>
+#else
+# include <linux/inotify.h>
+# include "inotify-syscalls.h"
+#endif
 
 #include <libintl.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+
+#include <X11/Xlib.h>
 
 #include <dbus/dbus.h>
 
 #include "gpomme.h"
 #include "theme.h"
-#include "audio.h"
 #include "conffile.h"
 
-#include "../dbus-client/dbus-client.h"
+#include "../client-common/dbus-client.h"
+#include "../client-common/video-client.h"
 
 
 #define _(str) gettext(str)
@@ -282,7 +292,7 @@ mbp_dbus_connect(void)
 
   signals = MBP_DBUS_SIG_LCD | MBP_DBUS_SIG_KBD
     | MBP_DBUS_SIG_VOL | MBP_DBUS_SIG_MUTE
-    | MBP_DBUS_SIG_EJECT;
+    | MBP_DBUS_SIG_EJECT | MBP_DBUS_SIG_VIDEO;
 
   conn = mbp_dbus_init(&dbus_err, signals);
 
@@ -312,6 +322,8 @@ mbp_dbus_listen(gpointer userdata)
   int who;
   double ratio;
 
+  Display *dpy;
+
   /* Disconnected, try to reconnect */
   if (conn == NULL)
     {
@@ -373,10 +385,7 @@ mbp_dbus_listen(gpointer userdata)
 	  ratio = (double)cur / (double)max;
 
 	  if (!mbp.muted)
-	    {
-	      show_window(IMG_AUDIO_VOL_ON, _("Sound volume"), ratio);
-	      audio_command(AUDIO_CLICK);
-	    }
+	    show_window(IMG_AUDIO_VOL_ON, _("Sound volume"), ratio);
 	  else
 	    show_window(IMG_AUDIO_VOL_OFF, _("Sound volume (muted)"), ratio);
 	}
@@ -395,6 +404,11 @@ mbp_dbus_listen(gpointer userdata)
 	{
 	  show_window(IMG_CD_EJECT, _("Eject"), -1.0);
 	}
+      else if (dbus_message_is_signal(msg, "org.pommed.signal.videoSwitch", "videoSwitch"))
+	{
+	  dpy = GDK_WINDOW_XDISPLAY(GTK_WIDGET(mbp_w.window)->window);
+	  mbp_video_switch(dpy);
+	}
       else if (dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
 	{
 	  printf("DBus disconnected\n");
@@ -450,7 +464,7 @@ mbp_check_events(gpointer userdata)
 static void
 usage(void)
 {
-  printf("gpomme v" M_VERSION " ($Rev: 341 $) graphical client for pommed\n");
+  printf("gpomme v" M_VERSION " ($Rev: 413 $) graphical client for pommed\n");
   printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org> and others\n");
 
   printf("Usage:\n");
@@ -465,6 +479,18 @@ void sig_int_term_handler(int signo)
   gtk_main_quit();
 }
 
+void
+sig_chld_handler(int signo)
+{
+  int ret;
+
+  do
+    {
+      ret = waitpid(-1, NULL, WNOHANG);
+    }
+  while (ret > 0);
+}
+
 int main(int argc, char **argv)
 {
   int c;
@@ -494,7 +520,7 @@ int main(int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("gpomme v" M_VERSION " ($Rev: 341 $) graphical client for pommed\n");
+	    printf("gpomme v" M_VERSION " ($Rev: 413 $) graphical client for pommed\n");
 	    printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org> and others\n");
 
 	    exit(0);
@@ -514,10 +540,7 @@ int main(int argc, char **argv)
 
   signal(SIGINT, sig_int_term_handler);
   signal(SIGTERM, sig_int_term_handler);
-
-  ret = audio_init_thread();
-  if (ret < 0)
-    printf("Failed to create audio thread\n");
+  signal(SIGCHLD, sig_chld_handler);
 
   create_window();
 
@@ -525,12 +548,8 @@ int main(int argc, char **argv)
 
   gtk_main();
 
-
   close(cfg_ifd);
 
-  audio_command(AUDIO_COMMAND_QUIT);
-  audio_cleanup();
-
   mbp_dbus_cleanup();
 
   return 0;
