@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: evdev.c 336 2007-06-17 08:44:45Z jblache $
+ * $Id: evdev.c 417 2007-12-07 12:39:59Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -26,12 +26,20 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
-#include <poll.h>
 
 #include <syslog.h>
 
 #include <errno.h>
 
+#include <sys/epoll.h>
+
+#ifndef NO_SYS_INOTIFY_H
+# include <sys/inotify.h>
+#else
+# include <linux/inotify.h>
+# include "inotify-syscalls.h"
+#endif
+
 #include <linux/input.h>
 
 #include "pommed.h"
@@ -41,6 +49,8 @@
 #include "lcd_backlight.h"
 #include "cd_eject.h"
 #include "audio.h"
+#include "video.h"
+#include "beep.h"
 
 
 #define BITS_PER_LONG (sizeof(long) * 8)
@@ -56,20 +66,79 @@
 #endif
 
 
-#define EVDEV_ADB_KBD       (1 << 0)
-#define EVDEV_ADB_BUTTONS   (1 << 1)
-#define EVDEV_USB_KBD1      (1 << 2)
-#define EVDEV_USB_KBD2      (1 << 3)
-#define EVDEV_APPLEIR       (1 << 5)
-#define EVDEV_MOUSEEMU      (1 << 6)
-#define EVDEV_SW_LID        (1 << 7)
+/* Open event devices */
+static int ev_fds[EVDEV_MAX];
 
-#define EVDEV_NO_EVDEV      0
+/* epoll fd */
+static int epfd;
 
-static unsigned int evdevs;
+/* inotify fd */
+static int ifd;
 
 
-void
+static int
+evdev_try_add(int fd);
+
+
+static int
+evdev_add(int fd)
+{
+  int i;
+  int ret;
+
+  struct epoll_event epoll_ev;
+
+  for (i = 0; i < EVDEV_MAX; i++)
+    {
+      if (ev_fds[i] == -1)
+	{
+	  ev_fds[i] = fd;
+
+	  break;
+	}
+    }
+
+  epoll_ev.events = EPOLLIN;
+  epoll_ev.data.fd = fd;
+
+  ret = epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &epoll_ev);
+
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Could not add device to epoll: %s", strerror(errno));
+
+      return -1;
+    }
+
+  return 0;
+}
+
+static void
+evdev_remove(int fd)
+{
+  int i;
+  int ret;
+
+  for (i = 0; i < EVDEV_MAX; i++)
+    {
+      if (ev_fds[i] == fd)
+	{
+	  ev_fds[i] = -1;
+
+	  break;
+	}
+    }
+
+  ret = epoll_ctl(epfd, EPOLL_CTL_DEL, fd, NULL);
+
+  if (ret < 0)
+    logmsg(LOG_ERR, "Could not remove device from epoll: %s", strerror(errno));
+
+  close(fd);
+}
+
+
+static void
 evdev_process_events(int fd)
 {
   int ret;
@@ -87,6 +156,10 @@ evdev_process_events(int fd)
       if (ev.value == 0)
 	return;
 
+      /* Reset keyboard backlight idle timer */
+      kbd_bck_info.idle = 0;
+      kbd_backlight_inhibit_clear(KBD_INHIBIT_IDLE);
+
       switch (ev.code)
 	{
 	  case K_LCD_BCK_DOWN:
@@ -121,6 +194,8 @@ evdev_process_events(int fd)
 
 	  case K_VIDEO_TOGGLE:
 	    logdebug("\nKEY: video toggle\n");
+
+	    video_switch();
 	    break;
 
 	  case K_KBD_BCK_OFF:
@@ -182,6 +257,16 @@ evdev_process_events(int fd)
 	    break;
 	}
     }
+  else if (ev.type == EV_SND)
+    {
+      /* Beeper device */
+      if ((ev.code == SND_TONE) && (ev.value > 0))
+	{
+	  logdebug("\nBEEP: BEEP!\n");
+
+	  beep_beep(); /* Catch that, Coyote */
+	}
+    }
   else if (ev.type == EV_SW)
     {
       /* Lid switch */
@@ -203,6 +288,154 @@ evdev_process_events(int fd)
     }
 }
 
+
+static void
+evdev_inotify_process(void)
+{
+  int ret;
+  int fd;
+  int qsize;
+
+  struct inotify_event *all_ie;
+  struct inotify_event *ie;
+  char evdev[32];
+
+  /* Determine the size of the inotify queue */
+  ret = ioctl(ifd, FIONREAD, &qsize);
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Could not determine inotify queue size: %s", strerror(errno));
+
+      return;
+    }
+
+  all_ie = (struct inotify_event *) malloc(qsize);
+  if (all_ie == NULL)
+    {
+      logmsg(LOG_ERR, "Could not allocate %d bytes for inotify events");
+
+      return;
+    }
+
+  ret = read(ifd, all_ie, qsize);
+  if (ret < 0)
+    {
+      logmsg(LOG_WARNING, "inotify read failed: %s", strerror(errno));
+
+      free(all_ie);
+      return;
+    }
+
+  /* ioctl(FIONREAD) returns the number of bytes, now we need the number of elements */
+  qsize /= sizeof(struct inotify_event);
+
+  /* Loop through all the events we got */
+  for (ie = all_ie; (ie - all_ie) < qsize; ie += (1 + (ie->len / sizeof(struct inotify_event))))
+    {
+      /* ie[0] contains the inotify event information
+       * the memory space for ie[1+] contains the name of the file
+       * see the inotify documentation
+       */
+
+      if ((ie->len == 0) || (ie->name == NULL))
+	{
+	  logdebug("inotify event with no name\n");
+
+	  continue;
+	}
+
+      logdebug("Found new event device %s/%s\n", EVDEV_DIR, ie->name);
+
+      if (strncmp("event", ie->name, 5))
+	{
+	  logdebug("Discarding %s/%s\n", EVDEV_DIR, ie->name);
+
+	  continue;
+	}
+
+      ret = snprintf(evdev, sizeof(evdev), "%s/%s", EVDEV_DIR, ie->name);
+
+      if ((ret <= 0) || (ret >= sizeof(evdev)))
+	continue;
+
+      fd = open(evdev, O_RDWR);
+      if (fd < 0)
+	{
+	  if (errno != ENOENT)
+	    logmsg(LOG_WARNING, "Could not open %s: %s", evdev, strerror(errno));
+
+	  continue;
+	}
+
+      evdev_try_add(fd);
+    }
+
+  free(all_ie);
+}
+
+
+int
+evdev_event_loop(void)
+{
+  int i;
+  int nfds;
+
+  int inotify = 0;
+
+  struct epoll_event events[MAX_EPOLL_EVENTS];
+
+  nfds = epoll_wait(epfd, events, MAX_EPOLL_EVENTS, LOOP_TIMEOUT);
+
+  if (nfds < 0)
+    {
+      if (errno == EINTR)
+	return 1; /* pommed.c will go on with the events management */
+      else
+	{
+	  logmsg(LOG_ERR, "epoll_wait() error: %s", strerror(errno));
+
+	  return -1; /* pommed.c will exit */
+	}
+    }
+
+
+  for (i = 0; i < nfds; i++)
+    {
+      /* some of the event devices cease to exist when suspending */
+      if (events[i].events & (EPOLLERR | EPOLLHUP))
+	{
+	  logmsg(LOG_INFO, "Error condition signaled on event device");
+
+	  if (events[i].data.fd == beep_info.fd)
+	    logmsg(LOG_WARNING, "Beeper device lost; this should not happen");
+
+	  if (events[i].data.fd == ifd)
+	    {
+	      logmsg(LOG_WARNING, "inotify fd lost; this should not happen");
+	      ifd = -1;
+	    }
+
+	  evdev_remove(events[i].data.fd);
+
+	  continue;
+	}
+
+      if (events[i].events & EPOLLIN)
+	{
+	  if (events[i].data.fd == ifd)
+	    inotify = 1; /* defer inotify events processing */
+	  else
+	    evdev_process_events(events[i].data.fd);
+	}
+    }
+
+  if (inotify)
+    evdev_inotify_process();
+
+  return nfds;
+}
+
+
 #ifdef __powerpc__
 /* PowerBook G4 Titanium */
 int
@@ -220,7 +453,6 @@ evdev_is_adb(unsigned short *id)
     {
       logdebug(" -> ADB keyboard\n");
 
-      evdevs |= EVDEV_ADB_KBD;
       return 1;
     }
 
@@ -228,7 +460,6 @@ evdev_is_adb(unsigned short *id)
     {
       logdebug(" -> ADB PowerBook buttons\n");
 
-      evdevs |= EVDEV_ADB_BUTTONS;
       return 1;
     }
 
@@ -253,11 +484,6 @@ evdev_is_fountain(unsigned short *id)
     {
       logdebug(" -> Fountain USB keyboard\n");
 
-      if (evdevs & EVDEV_USB_KBD1)
-	evdevs |= EVDEV_USB_KBD2;
-      else
-	evdevs |= EVDEV_USB_KBD1;
-
       return 1;
     }
 
@@ -281,10 +507,7 @@ evdev_is_geyser(unsigned short *id)
     {
       logdebug(" -> Geyser USB keyboard\n");
 
-      if (evdevs & EVDEV_USB_KBD1)
-	evdevs |= EVDEV_USB_KBD2;
-      else
-	evdevs |= EVDEV_USB_KBD1;
+      kbd_set_fnmode();
 
       return 1;
     }
@@ -311,7 +534,6 @@ evdev_is_lidswitch(unsigned short *id)
     {
       logdebug(" -> PMU LID switch\n");
 
-      evdevs |= EVDEV_SW_LID;
       return 1;
     }
 
@@ -338,10 +560,7 @@ evdev_is_geyser3(unsigned short *id)
     {
       logdebug(" -> Geyser III USB keyboard\n");
 
-      if (evdevs & EVDEV_USB_KBD1)
-	evdevs |= EVDEV_USB_KBD2;
-      else
-	evdevs |= EVDEV_USB_KBD1;
+      kbd_set_fnmode();
 
       return 1;
     }
@@ -367,10 +586,33 @@ evdev_is_geyser4(unsigned short *id)
     {
       logdebug(" -> Geyser IV USB keyboard\n");
 
-      if (evdevs & EVDEV_USB_KBD1)
-	evdevs |= EVDEV_USB_KBD2;
-      else
-	evdevs |= EVDEV_USB_KBD1;
+      kbd_set_fnmode();
+
+      return 1;
+    }
+
+  return 0;
+}
+
+/* Core2 Duo Santa Rosa MacBook (MacBook3,1) */
+int
+evdev_is_geyser4hf(unsigned short *id)
+{
+  unsigned short product = id[ID_PRODUCT];
+
+  if (id[ID_BUS] != BUS_USB)
+    return 0;
+
+  if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
+    return 0;
+
+  if ((product == USB_PRODUCT_ID_GEYSER4HF_ANSI)
+      || (product == USB_PRODUCT_ID_GEYSER4HF_ISO)
+      || (product == USB_PRODUCT_ID_GEYSER4HF_JIS))
+    {
+      logdebug(" -> Geyser IV-HF USB keyboard\n");
+
+      kbd_set_fnmode();
 
       return 1;
     }
@@ -395,7 +637,6 @@ evdev_is_appleir(unsigned short *id)
     {
       logdebug(" -> Apple IR receiver\n");
 
-      evdevs |= EVDEV_APPLEIR;
       return 1;
     }
 
@@ -418,7 +659,6 @@ evdev_is_lidswitch(unsigned short *id)
     {
       logdebug(" -> ACPI LID switch\n");
 
-      evdevs |= EVDEV_SW_LID;
       return 1;
     }
 
@@ -426,6 +666,31 @@ evdev_is_lidswitch(unsigned short *id)
 }
 #endif /* !__powerpc__ */
 
+/* External Apple USB keyboards */
+int
+evdev_is_extkbd(unsigned short *id)
+{
+  unsigned short product = id[ID_PRODUCT];
+
+  if (id[ID_BUS] != BUS_USB)
+    return 0;
+
+  if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
+    return 0;
+
+  if ((product == USB_PRODUCT_ID_APPLE_EXTKBD_WHITE)
+      || (product == USB_PRODUCT_ID_APPLE_EXTKBD_ALU))
+    {
+      logdebug(" -> External Apple USB keyboard\n");
+
+      kbd_set_fnmode();
+
+      return 1;
+    }
+
+  return 0;
+}
+
 /* Mouseemu virtual keyboard */
 static int
 evdev_is_mouseemu(unsigned short *id)
@@ -442,7 +707,6 @@ evdev_is_mouseemu(unsigned short *id)
     {
       logdebug(" -> Mouseemu virtual keyboard\n");
 
-      evdevs |= EVDEV_MOUSEEMU;
       return 1;
     }
 
@@ -450,22 +714,136 @@ evdev_is_mouseemu(unsigned short *id)
 }
 
 
-int
-evdev_open(struct pollfd **fds)
+static int
+evdev_try_add(int fd)
 {
-  int ret;
-  int i, j;
-
-  int found = 0;
-  int fd[32];
-
   unsigned short id[4];
   unsigned long bit[EV_MAX][NBITS(KEY_MAX)];
   char devname[256];
+
+  int ret;
+
+  devname[0] = '\0';
+  ioctl(fd, EVIOCGNAME(sizeof(devname)), devname);
+
+  logdebug("\nInvestigating evdev [%s]\n", devname);
+
+  ioctl(fd, EVIOCGID, id);
+
+  if ((!mops->evdev_identify(id))
+#ifndef __powerpc__
+      && !(appleir_cfg.enabled && evdev_is_appleir(id))
+#endif
+      && !(has_kbd_backlight() && evdev_is_lidswitch(id))
+      && !(evdev_is_mouseemu(id))
+      && !(evdev_is_extkbd(id)))
+    {
+      logdebug("Discarding evdev: bus 0x%04x, vid 0x%04x, pid 0x%04x\n", id[ID_BUS], id[ID_VENDOR], id[ID_PRODUCT]);
+
+      close(fd);
+
+      return -1;
+    }
+
+  memset(bit, 0, sizeof(bit));
+
+  ioctl(fd, EVIOCGBIT(0, EV_MAX), bit[0]);
+
+  if (!test_bit(EV_KEY, bit[0]))
+    {
+      logdebug("evdev: no EV_KEY event type (not a keyboard)\n");
+
+      if (!test_bit(EV_SW, bit[0]))
+	{
+	  logdebug("Discarding evdev: no EV_SW event type (not a switch)\n");
+
+	  close(fd);
+
+	  return -1;
+	}
+    }
+  else if (test_bit(EV_ABS, bit[0]))
+    {
+      logdebug("Discarding evdev with EV_ABS event type (mouse/trackpad)\n");
+
+      close(fd);
+
+      return -1;
+    }
+
+  ret = evdev_add(fd);
+
+  return ret;
+}
+
+
+static int
+evdev_inotify_init(void)
+{
+  int ret;
+  struct epoll_event epoll_ev;
+
+  ifd = inotify_init();
+  if (ifd < 0)
+    {
+      logmsg(LOG_ERR, "Failed to initialize inotify: %s", strerror(errno));
+
+      return -1;
+    }
+
+  ret = inotify_add_watch(ifd, EVDEV_DIR, IN_CREATE | IN_ONLYDIR);
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Failed to add inotify watch for %s: %s", EVDEV_DIR, strerror(errno));
+
+      close(ifd);
+      ifd = -1;
+
+      return -1;
+    }
+
+  epoll_ev.events = EPOLLIN;
+  epoll_ev.data.fd = ifd;
+
+  ret = epoll_ctl(epfd, EPOLL_CTL_ADD, ifd, &epoll_ev);
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Failed to add inotify fd to epoll: %s", strerror(errno));
+
+      close(ifd);
+
+      return -1;
+    }
+
+  return 0;
+}
+
+
+int
+evdev_init(void)
+{
+  int ret;
+  int i;
+
   char evdev[32];
 
-  evdevs = EVDEV_NO_EVDEV;
+  int ndevs;
+  int fd;
 
+  epfd = epoll_create(MAX_EPOLL_EVENTS);
+  if (epfd < 0)
+    {
+      logmsg(LOG_ERR, "Could not create epoll fd: %s", strerror(errno));
+
+      return -1;
+    }
+
+  for (i = 0; i < EVDEV_MAX; i++)
+    {
+      ev_fds[i] = -1;
+    }
+
+  ndevs = 0;
   for (i = 0; i < EVDEV_MAX; i++)
     {
       ret = snprintf(evdev, 32, "%s%d", EVDEV_BASE, i);
@@ -473,8 +851,8 @@ evdev_open(struct pollfd **fds)
       if ((ret <= 0) || (ret > 31))
 	return -1;
 
-      fd[i] = open(evdev, O_RDWR);
-      if (fd[i] < 0)
+      fd = open(evdev, O_RDWR);
+      if (fd < 0)
 	{
 	  if (errno != ENOENT)
 	    logmsg(LOG_WARNING, "Could not open %s: %s", evdev, strerror(errno));
@@ -482,140 +860,42 @@ evdev_open(struct pollfd **fds)
 	  continue;
 	}
 
-      devname[0] = '\0';
-      ioctl(fd[i], EVIOCGNAME(sizeof(devname)), devname);
-      logdebug("\nInvestigating evdev %d [%s]\n", i, devname);
-
-
-      ioctl(fd[i], EVIOCGID, id);
-
-      if ((!mops->evdev_identify(id))
-#ifndef __powerpc__
-	  && !(appleir_cfg.enabled && evdev_is_appleir(id))
-#endif
-	  && !(has_kbd_backlight() && evdev_is_lidswitch(id))
-	  && !(evdev_is_mouseemu(id)))
-	{
-	  logdebug("Discarding evdev %d: bus 0x%04x, vid 0x%04x, pid 0x%04x\n", i, id[ID_BUS], id[ID_VENDOR], id[ID_PRODUCT]);
-
-	  close(fd[i]);
-	  fd[i] = -1;
-
-	  continue;
-	}
-
-      memset(bit, 0, sizeof(bit));
-
-      ioctl(fd[i], EVIOCGBIT(0, EV_MAX), bit[0]);
-
-      if (!test_bit(EV_KEY, bit[0]))
-	{
-	  logdebug("evdev %d: no EV_KEY event type (not a keyboard)\n", i);
-
-	  if (!test_bit(EV_SW, bit[0]))
-	    {
-	      logdebug("evdev %d: no EV_SW event type (not a switch)\n", i);
-
-	      close(fd[i]);
-	      fd[i] = -1;
-
-	      logdebug("Discarding evdev %d\n", i);
-
-	      continue;
-	    }
-	}
-      else if (test_bit(EV_ABS, bit[0]))
-	{
-	  logdebug("Discarding evdev %d with EV_ABS event type (mouse/trackpad)\n", i);
-
-	  close(fd[i]);
-	  fd[i] = -1;
-
-	  continue;
-	}
-
-      found++;
+      if (evdev_try_add(fd) == 0)
+	ndevs++;
     }
 
-  logdebug("\nFound %d devices\n", found);
+  logdebug("\nFound %d devices\n", ndevs);
 
-  *fds = (struct pollfd *) malloc(found * sizeof(struct pollfd));
-
-  if (*fds == NULL)
+  /* Add the beeper device */
+  ret = beep_open_device();
+  if (ret == 0)
     {
-      for (i = 0; i < EVDEV_MAX; i++)
-	{
-	  if (fd[i] > 0)
-	    close(fd[i]);
-	}
+      ret = evdev_add(beep_info.fd);
 
-      logmsg(LOG_ERR, "Out of memory for %d pollfd structs", found);
-
-      return -1;
+      if (ret == 0)
+	  ndevs++;
     }
 
-  j = 0;
-  for (i = 0; i < EVDEV_MAX && j < found; i++)
-    {
-      if (fd[i] < 0)
-	continue;
+  /* Initialize inotify */
+  evdev_inotify_init();
 
-      (*fds)[j].fd = fd[i];
-      (*fds)[j].events = POLLIN;
-      j++;
-    }
-
-  return found;
+  return ndevs;
 }
 
 void
-evdev_close(struct pollfd **fds, int nfds)
+evdev_cleanup(void)
 {
   int i;
 
-  if (*fds != NULL)
+  close(epfd);
+
+  close(ifd);
+
+  for (i = 0; i < EVDEV_MAX; i++)
     {
-      for (i = 0; i < nfds; i++)
-	close((*fds)[i].fd);
-
-      free(*fds);
+      if (ev_fds[i] == beep_info.fd)
+	beep_close_device();
+      else
+	close(ev_fds[i]);
     }
-
-  *fds = NULL;
-}
-
-
-int
-evdev_reopen(struct pollfd **fds, int nfds)
-{
-  int i;
-  unsigned int prev_evdevs = evdevs;
-
-  evdev_close(fds, nfds);
-
-  logdebug("Previous event devices: 0x%04x\n", prev_evdevs);
-
-  /* When resuming, we need to reopen event devices which
-   * disappear at suspend time. We need to wait for udev to
-   * recreate the device nodes.
-   */
-  for (i = 0; i < 50; i++)
-    {
-      usleep(500000);
-
-      nfds = evdev_open(fds);
-
-      if (nfds > 0)
-	{
-	  logdebug("Got event devices 0x%04x at iteration %d\n", prev_evdevs, i);
-
-	  if (evdevs == prev_evdevs)
-	    break;
-
-	  /* We haven't got all the event devices we need */
-	  evdev_close(fds, nfds);
-	}
-    }
-
-  return nfds;
 }
