@@ -1,5 +1,5 @@
 /*
- * $Id: evdev.h 336 2007-06-17 08:44:45Z jblache $
+ * $Id: evdev.h 377 2007-11-16 18:44:29Z jblache $
  */
 
 #ifndef __EVDEV_H__
@@ -38,6 +38,15 @@
 #define USB_PRODUCT_ID_GEYSER4_ISO    0x021b
 #define USB_PRODUCT_ID_GEYSER4_JIS    0x021c
 
+/* Apple Geyser IV-HF keyboard + trackpad */
+#define USB_PRODUCT_ID_GEYSER4HF_ANSI   0x0229
+#define USB_PRODUCT_ID_GEYSER4HF_ISO    0x022a
+#define USB_PRODUCT_ID_GEYSER4HF_JIS    0x022b
+
+/* Apple USB keyboards, external */
+#define USB_PRODUCT_ID_APPLE_EXTKBD_WHITE   0x020c
+#define USB_PRODUCT_ID_APPLE_EXTKBD_ALU     0x0221
+
 /* Apple Remote IR Receiver */
 #define USB_PRODUCT_ID_APPLEIR        0x8240
 #define USB_PRODUCT_ID_APPLEIR_2      0x8242
@@ -63,12 +72,15 @@
 #define K_IR_MENU               0x8b
 
 
+#define EVDEV_DIR               "/dev/input"
 #define EVDEV_BASE              "/dev/input/event"
 #define EVDEV_MAX               32
 
+#define MAX_EPOLL_EVENTS        8
 
-void
-evdev_process_events(int fd);
+
+int
+evdev_event_loop(void);
 
 
 #ifdef __powerpc__
@@ -88,16 +100,16 @@ evdev_is_geyser3(unsigned short *id);
 
 int
 evdev_is_geyser4(unsigned short *id);
+
+int
+evdev_is_geyser4hf(unsigned short *id);
 #endif /* __powerpc__ */
 
 
 int
-evdev_open(struct pollfd **fds);
+evdev_init(void);
 
 void
-evdev_close(struct pollfd **fds, int nfds);
-
-int
-evdev_reopen(struct pollfd **fds, int nfds);
+evdev_cleanup(void);
 
 #endif /* !__EVDEV_H__ */
