@@ -1,5 +1,5 @@
 /*
- * $Id: kbd_backlight.h 319 2007-05-03 08:28:44Z jblache $
+ * $Id: kbd_backlight.h 432 2008-01-23 13:33:31Z jblache $
  */
 
 #ifndef __KBD_BACKLIGHT_H__
@@ -20,6 +20,9 @@
 #define KBD_INHIBIT_USER        (1 << 0)
 #define KBD_INHIBIT_LID         (1 << 1)
 #define KBD_INHIBIT_CFG         (1 << 2)
+#define KBD_INHIBIT_IDLE        (1 << 3)
+
+#define KBD_MASK_AUTO           (KBD_INHIBIT_LID | KBD_INHIBIT_IDLE)
 
 
 #define KBD_USER     0
@@ -37,15 +40,40 @@ struct _kbd_bck_info
   int toggle_lvl; /* backlight level for simple toggle */
 
   int auto_on;  /* automatic */
+  int idle;     /* idle timer */
   int r_sens;   /* right sensor */
   int l_sens;   /* left sensor */
 };
 
 extern struct _kbd_bck_info kbd_bck_info;
 
+#ifdef __powerpc__
+static inline int
+has_kbd_backlight(void)
+{
+  return ((mops->type == MACHINE_POWERBOOK_51)
+	  || (mops->type == MACHINE_POWERBOOK_52)
+	  || (mops->type == MACHINE_POWERBOOK_53)
+	  || (mops->type == MACHINE_POWERBOOK_54)
+	  || (mops->type == MACHINE_POWERBOOK_55)
+	  || (mops->type == MACHINE_POWERBOOK_56)
+	  || (mops->type == MACHINE_POWERBOOK_57)
+	  || (mops->type == MACHINE_POWERBOOK_58)
+	  || (mops->type == MACHINE_POWERBOOK_59));
+}
+
+#else
+
+static inline int
+has_kbd_backlight(void)
+{
+  return ((mops->type == MACHINE_MACBOOKPRO_1)
+	  || (mops->type == MACHINE_MACBOOKPRO_2)
+	  || (mops->type == MACHINE_MACBOOKPRO_3)
+	  || (mops->type == MACHINE_MACBOOKAIR_1));
+}
+#endif /* __powerpc__ */
 
-int
-has_kbd_backlight(void);
 
 void
 kbd_backlight_step(int dir);
