@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: kbd_backlight.c 336 2007-06-17 08:44:45Z jblache $
+ * $Id: kbd_backlight.c 352 2007-08-22 14:14:16Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -42,15 +42,6 @@
 struct _kbd_bck_info kbd_bck_info;
 
 
-int
-has_kbd_backlight(void)
-{
-  return ((mops->type == MACHINE_MACBOOKPRO_1)
-	  || (mops->type == MACHINE_MACBOOKPRO_2)
-	  || (mops->type == MACHINE_MACBOOKPRO_3));
-}
-
-
 static int
 kbd_backlight_get(void)
 {
@@ -246,4 +237,7 @@ kbd_backlight_fix_config(void)
 
   if (kbd_cfg.step > (KBD_BACKLIGHT_MAX / 2))
     kbd_cfg.step = KBD_BACKLIGHT_MAX / 2;
+
+  if (kbd_cfg.idle > 0)
+    kbd_cfg.idle = (kbd_cfg.idle * 1000) / LOOP_TIMEOUT;
 }
