@@ -1,9 +1,9 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: pommed.c 345 2007-07-27 14:00:46Z jblache $
+ * $Id: pommed.c 434 2008-02-08 20:36:15Z jblache $
  *
- * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <sys/time.h>
 #include <time.h>
 #include <string.h>
-#include <poll.h>
 #include <signal.h>
 
 #include <syslog.h>
@@ -54,6 +53,7 @@
 #include "audio.h"
 #include "dbus.h"
 #include "power.h"
+#include "beep.h"
 
 
 /* Machine-specific operations */
@@ -283,6 +283,23 @@ struct machine_ops mb_mops[] = {
     .lcd_backlight_step = gma950_backlight_step,
     .lcd_backlight_toggle = gma950_backlight_toggle,
     .evdev_identify = evdev_is_geyser4,
+  },
+
+  {  /* MacBook3,1 (Core2 Duo Santa Rosa, November 2007) */
+    .type = MACHINE_MACBOOK_3,
+    .lcd_backlight_probe = gma950_backlight_probe, /* gma950 supports the gma965 */
+    .lcd_backlight_step = gma950_backlight_step,
+    .lcd_backlight_toggle = gma950_backlight_toggle,
+    .evdev_identify = evdev_is_geyser4hf,
+  },
+
+  /* MacBook Air machines */
+  {  /* MacBook Air1,1 (January 2008) */
+    .type = MACHINE_MACBOOKAIR_1,
+    .lcd_backlight_probe = gma950_backlight_probe, /* gma950 supports the gma965 */
+    .lcd_backlight_step = gma950_backlight_step,
+    .lcd_backlight_toggle = gma950_backlight_toggle,
+    .evdev_identify = evdev_is_geyser4hf,
   }
 };
 #endif /* __powerpc__ */
@@ -567,6 +584,12 @@ check_machine_smbios(void)
   /* Core2 Duo MacBook (November 2006) */
   else if (strcmp(prop, "MacBook2,1") == 0)
     ret = MACHINE_MACBOOK_2;
+  /* Core2 Duo Santa Rosa MacBook (November 2007) */
+  else if (strcmp(prop, "MacBook3,1") == 0)
+    ret = MACHINE_MACBOOK_3;
+  /* MacBook Air (January 2008) */
+  else if (strcmp(prop, "MacBook Air1,1") == 0)
+    ret = MACHINE_MACBOOKAIR_1;
   else
     logmsg(LOG_ERR, "Unknown Apple machine: %s", prop);
 
@@ -583,8 +606,8 @@ check_machine_smbios(void)
 static void
 usage(void)
 {
-  printf("pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler\n");
-  printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>\n");
+  printf("pommed v" M_VERSION " ($Rev: 434 $) Apple laptops hotkeys handler\n");
+  printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
 
   printf("Usage:\n");
   printf("\tpommed\t-- start pommed as a daemon\n");
@@ -605,14 +628,9 @@ main (int argc, char **argv)
 {
   int ret;
   int c;
-  int i;
 
   FILE *pidfile;
 
-  struct pollfd *fds;
-  int nfds;
-
-  int reopen;
   machine_type machine;
 
   struct timeval tv_now;
@@ -633,8 +651,8 @@ main (int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler\n");
-	    printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>\n");
+	    printf("pommed v" M_VERSION " ($Rev: 434 $) Apple laptops hotkeys handler\n");
+	    printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
 
 	    exit(0);
 	    break;
@@ -652,8 +670,8 @@ main (int argc, char **argv)
       openlog("pommed", LOG_PID, LOG_DAEMON);
     }
 
-  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler");
-  logmsg(LOG_INFO, "Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>");
+  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 434 $) Apple laptops hotkeys handler");
+  logmsg(LOG_INFO, "Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>");
 
   /* Load our configuration */
   ret = config_load();
@@ -705,20 +723,19 @@ main (int argc, char **argv)
   ret = mops->lcd_backlight_probe();
   if (ret < 0)
     {
-      logmsg(LOG_ERR, "No LCD backlight found");
+      logmsg(LOG_ERR, "LCD backlight probe failed, check debug output");
 
       exit(1);
     }
 
-  nfds = evdev_open(&fds);
-  if (nfds < 1)
+  ret = evdev_init();
+  if (ret < 1)
     {
       logmsg(LOG_ERR, "No suitable event devices found");
 
       exit(1);
     }
 
-  kbd_set_fnmode();
   kbd_backlight_init();
 
   ret = audio_init();
@@ -744,7 +761,7 @@ main (int argc, char **argv)
 	{
 	  logmsg(LOG_ERR, "daemon() failed: %s", strerror(errno));
 
-	  evdev_close(&fds, nfds);
+	  evdev_cleanup();
 
 	  exit(-1);
 	}
@@ -755,73 +772,36 @@ main (int argc, char **argv)
     {
       logmsg(LOG_WARNING, "Could not open pidfile %s: %s", PIDFILE, strerror(errno));
 
-      evdev_close(&fds, nfds);
+      evdev_cleanup();
 
       exit(-1);
     }
   fprintf(pidfile, "%d\n", getpid());
   fclose(pidfile);
 
+  /* Spawn the beep thread */
+  beep_init();
+
   gettimeofday(&tv_als, NULL);
 
   running = 1;
   signal(SIGINT, sig_int_term_handler);
   signal(SIGTERM, sig_int_term_handler);
 
-  reopen = 0;
 
   while (running)
     {
-      /* Attempt to reopen event devices, typically after resuming */
-      if (reopen)
-	{
-	  nfds = evdev_reopen(&fds, nfds);
+      ret = evdev_event_loop();
 
-	  if (nfds < 1)
-	    {
-	      logmsg(LOG_ERR, "No suitable event devices found (reopen)");
-
-	      break;
-	    }
-
-	  /* Re-set the keyboard mode
-	   * When we need to reopen the event devices, it means we've
-	   * just resumed from sleep
-	   */
-	  kbd_set_fnmode();
-
-	  reopen = 0;
-	}
-
-      ret = poll(fds, nfds, LOOP_TIMEOUT);
+      gettimeofday(&tv_now, NULL);
 
       if (ret < 0) /* error */
 	{
-	  if (errno != EINTR)
-	    {
-	      logmsg(LOG_ERR, "poll() error: %s", strerror(errno));
-
-	      break;
-	    }
+	  break;
 	}
       else if (ret != 0)
 	{
-	  for (i = 0; i < nfds; i++)
-	    {
-	      /* the event devices cease to exist when suspending */
-	      if (fds[i].revents & (POLLERR | POLLHUP | POLLNVAL))
-		{
-		  logmsg(LOG_WARNING, "Error condition signaled on evdev, reopening");
-		  reopen = 1;
-		  break;
-		}
-
-	      if (fds[i].revents & POLLIN)
-		evdev_process_events(fds[i].fd);
-	    }
-
 	  /* is it time to chek the ambient light sensors and AC state ? */
-	  gettimeofday(&tv_now, NULL);
 	  tv_diff.tv_sec = tv_now.tv_sec - tv_als.tv_sec;
 	  if (tv_diff.tv_sec < 0)
 	    tv_diff.tv_sec = 0;
@@ -839,34 +819,35 @@ main (int argc, char **argv)
 
 	  if (tv_diff.tv_usec >= (1000 * LOOP_TIMEOUT))
 	    {
-	      if (has_kbd_backlight())
-		{
-		  kbd_backlight_ambient_check();
-		}
-
-	      power_check_ac_state();
-
-	      tv_als = tv_now;
+	      ret = 0; /* go and check ALS, AC state and idle time */
 	    }
 	}
-      else
+
+      if (ret == 0)
 	{
-	  /* poll() timed out, check ambient light sensors and AC state */
+	  /* time to check ambient light sensors, AC state and idle time */
 	  if (has_kbd_backlight())
 	    {
+	      /* Increment keyboard backlight idle timer */
+	      kbd_bck_info.idle++;
+	      if ((kbd_cfg.idle > 0) && (kbd_bck_info.idle > kbd_cfg.idle))
+		kbd_backlight_inhibit_set(KBD_INHIBIT_IDLE);
+
 	      kbd_backlight_ambient_check();
 	    }
 
 	  power_check_ac_state();
 
-	  gettimeofday(&tv_als, NULL);
+	  tv_als = tv_now;
 	}
 
       /* Process DBus requests */
       mbpdbus_process_requests();
     }
 
-  evdev_close(&fds, nfds);
+  evdev_cleanup();
+
+  beep_cleanup();
 
   mbpdbus_cleanup();
 
