@@ -1,12 +1,12 @@
 /*
- * $Id: pommed.h 340 2007-07-26 15:23:27Z jblache $
+ * $Id: pommed.h 437 2008-02-09 08:05:44Z jblache $
  */
 
 #ifndef __POMMED_H__
 #define __POMMED_H__
 
 
-#define M_VERSION "1.8"
+#define M_VERSION "1.15"
 
 
 extern int debug;
@@ -33,8 +33,12 @@ typedef enum
     MACHINE_MACBOOKPRO_1,
     MACHINE_MACBOOKPRO_2,
     MACHINE_MACBOOKPRO_3,
+
     MACHINE_MACBOOK_1,
     MACHINE_MACBOOK_2,
+    MACHINE_MACBOOK_3,
+
+    MACHINE_MACBOOKAIR_1,
 #else
     MACHINE_POWERBOOK_32,
     MACHINE_POWERBOOK_33,
